% Regolatore tempo-continuo
Rc = tf(10*[1/50 1],[1/(500)^2 .3/500 1]);

% Periodo di campionamento
T = 2e-3;

% Regolatore discreto - Eulero in avanti
Rd1 = tf(5e4*T*[1 50*T-1],[1 0.3*T*500-2 T^2*500^2-0.3*T*500+1],T);

% Regolatore discreto - Eulero all'indietro
Rd2 = tf(5e4*T*[1+50*T -1 0],[1+500^2*T^2+500*T*0.3 -2-500*T*0.3 1],T);

% Regolatore discreto - Tustin
Rd3 = c2d(Rc,T,'tustin');

bode(Rc,Rd1,Rd2,Rd3)

legend({'R_c(s)','Eulero in avanti','Eulero all''indietro','Tustin'})

pause
% Tustin con prewarping a w = 500 rad/s
Rd4 = c2d(Rc,T,'prewarp',500);

figure
bode(Rc,Rd3,Rd4)

legend({'R_c(s)','Tustin','Prewarping'})

pause

% Metodo dell'invarianza all'impulso
T  = 1e-3;
% Sistema tempo-continuo
sys_c = tf(10,[1 1 0]);
% Sistema tempo-discreto
sys_d = tf([10-10*exp(-T) 0],[conv([1 -1],[1 -exp(-T)])],T)

sys_d2 = minreal(c2d(sys_c,T,'imp'))

figure
impulse(sys_c,sys_d)
legend({'R_c(s)','Ivarianza all''impulso'})

pause
% Metodo della corrispondenza poli/zeri
T  = 1e-3;
% Sistema tempo-continuo
sys_c = tf(10*[1 5],[10 11 1]);
% Sistema tempo-discreto
sys_d = zpk([-1 exp(T*-5)],[exp(-0.1*T) exp(-T)],dcgain(sys_c)*(1-exp(-T))*(1-exp(-0.1*T))/(2*(1-exp(-5*T))),T);
% Sistema tempo-discreto - confronto matlab
sys_d2 = zpk([exp(T*-5)],[exp(-0.1*T) exp(-T)],dcgain(sys_c)*(1-exp(-T))*(1-exp(-0.1*T))/((1-exp(-5*T))),T);
sys_d3 = c2d(sys_c,T,'matched');

figure
step(sys_c,sys_d)
legend({'R_c(s)','Poli/zeri'})